
# q-expansion of newform 836.2.i.b, downloaded from the LMFDB on 29 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 836
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[419, [1, 0]], [761, [1, 0]], [705, [0, -1]]]
aps_data = [[0, 0], [-3, 3], [1, -1], [-4, 0], [1, 0], [0, -1], [3, -3], [5, -2], [0, -1], [0, -9], [0, 0], [2, 0], [3, -3], [-1, 1], [0, 11], [0, -3], [9, -9], [0, -1], [0, -1], [9, -9], [11, -11], [3, -3], [-12, 0], [0, -11], [17, -17], [0, 3], [-16, 0], [12, 0], [-13, 13], [6, 0], [0, -19], [-1, 1], [0, 17], [0, -7], [15, -15], [0, 0], [-7, 7], [-24, 0], [0, -23], [-5, 5], [-4, 0], [0, -23], [-12, 0], [-21, 21], [6, 0], [0, 15], [27, -27], [1, -1], [8, 0], [-22, 0], [15, -15], [0, 0], [0, -5], [0, 15], [0, 13], [-21, 21], [-11, 11], [15, -15], [-10, 0], [0, 19], [23, -23], [-2, 0], [-1, 1], [8, 0], [0, 21], [0, 33], [0, 0], [-13, 13], [15, -15], [-26, 0], [-14, 0], [35, -35], [0, -29], [38, 0], [-28, 0], [-15, 15], [0, -3], [-11, 11], [37, -37], [0, 7], [0, 0], [-35, 35], [0, 29], [0, -19], [27, -27], [0, 39], [18, 0], [22, 0], [7, -7], [8, 0], [-28, 0], [0, 25], [-12, 0], [-25, 25], [-39, 39], [0, -19], [0, -3], [-10, 0], [0, 5], [0, 35], [0, -7], [0, 11], [-44, 0], [26, 0], [12, 0], [46, 0], [-19, 19], [0, -45], [0, 15], [-10, 0], [-8, 0], [11, -11], [0, 33], [-4, 0], [0, 23], [-3, 3], [9, -9], [28, 0], [18, 0], [0, 1], [0, 13], [14, 0], [-6, 0], [4, 0], [-28, 0], [23, -23], [0, -35], [41, -41], [-23, 23], [-30, 0], [-25, 25], [-21, 21], [0, -45], [9, -9], [-14, 0], [0, -21], [0, -31], [8, 0], [-2, 0], [-6, 0], [0, -11], [0, 31], [0, -13], [0, -27], [-38, 0], [-3, 3], [7, -7], [47, -47], [0, 35], [24, 0], [-1, 1], [30, 0], [0, -57], [0, -15], [-7, 7], [40, 0], [-16, 0], [-35, 35], [0, 3], [0, -17], [-27, 27], [-53, 53], [-13, 13], [-15, 15], [2, 0], [29, -29], [29, -29], [0, 15]]
